(*auxiliary functions for processing xml:*)
let aod elem attr default =
	let attrib_declared attr elem =
		List.mem attr (fst (List.split (Xml.attribs elem)))
	in
	if attrib_declared attr elem then
		Xml.attrib elem attr
	else
		default
;;

(** [rng_of_xml elem]: extract random number generater of an XML node [elem]: *)
let rng_of_xml elem =
	let rngtype = 
		match (aod elem "RandomNumberGenerator" "MT19937") with
		"BOROSH13" -> Gsl_rng.BOROSH13
		| "COVEYOU" -> Gsl_rng.COVEYOU
		| "CMRG" -> Gsl_rng.CMRG
		| "FISHMAN18" -> Gsl_rng.FISHMAN18
		| "FISHMAN20" -> Gsl_rng.FISHMAN20
		| "FISHMAN2X" -> Gsl_rng.FISHMAN2X
		| "GFSR4" -> Gsl_rng.GFSR4
		| "KNUTHRAN" -> Gsl_rng.KNUTHRAN
		| "KNUTHRAN2" -> Gsl_rng.KNUTHRAN2
		| "LECUYER21" -> Gsl_rng.LECUYER21
		| "MINSTD" -> Gsl_rng.MINSTD
		| "MRG" -> Gsl_rng.MRG
		| "MT19937" -> Gsl_rng.MT19937
		| "MT19937_1999" -> Gsl_rng.MT19937_1999
		| "MT19937_1998" -> Gsl_rng.MT19937_1998
		| "R250" -> Gsl_rng.R250
		| "RAN0" -> Gsl_rng.RAN0
		| "RAN1" -> Gsl_rng.RAN1
		| "RAN2" -> Gsl_rng.RAN2
		| "RAN3" -> Gsl_rng.RAN3
		| "RAND" -> Gsl_rng.RAND
		| "RAND48" -> Gsl_rng.RAND48
		| "RANDOM128_BSD" -> Gsl_rng.RANDOM128_BSD
		| "RANDOM128_GLIBC2" -> Gsl_rng.RANDOM128_GLIBC2
		| "RANDOM128_LIBC5" -> Gsl_rng.RANDOM128_LIBC5
		| "RANDOM256_BSD" -> Gsl_rng.RANDOM256_BSD
		| "RANDOM256_GLIBC2" -> Gsl_rng.RANDOM256_GLIBC2
		| "RANDOM256_LIBC5" -> Gsl_rng.RANDOM256_LIBC5
		| "RANDOM32_BSD" -> Gsl_rng.RANDOM32_BSD
		| "RANDOM32_GLIBC2" -> Gsl_rng.RANDOM32_GLIBC2
		| "RANDOM32_LIBC5" -> Gsl_rng.RANDOM32_LIBC5
		| "RANDOM64_BSD" -> Gsl_rng.RANDOM64_BSD
		| "RANDOM64_GLIBC2" -> Gsl_rng.RANDOM64_GLIBC2
		| "RANDOM64_LIBC5" -> Gsl_rng.RANDOM64_LIBC5
		| "RANDOM8_BSD" -> Gsl_rng.RANDOM8_BSD
		| "RANDOM8_GLIBC2" -> Gsl_rng.RANDOM8_GLIBC2
		| "RANDOM8_LIBC5" -> Gsl_rng.RANDOM8_LIBC5
		| "RANDOM_BSD" -> Gsl_rng.RANDOM_BSD
		| "RANDOM_GLIBC2" -> Gsl_rng.RANDOM_GLIBC2
		| "RANDOM_LIBC5" -> Gsl_rng.RANDOM_LIBC5
		| "RANDU" -> Gsl_rng.RANDU
		| "RANF" -> Gsl_rng.RANF
		| "RANLUX" -> Gsl_rng.RANLUX
		| "RANLUX389" -> Gsl_rng.RANLUX389
		| "RANLXD1" -> Gsl_rng.RANLXD1
		| "RANLXD2" -> Gsl_rng.RANLXD2
		| "RANLXS0" -> Gsl_rng.RANLXS0
		| "RANLXS1" -> Gsl_rng.RANLXS1
		| "RANLXS2" -> Gsl_rng.RANLXS2
		| "RANMAR" -> Gsl_rng.RANMAR
		| "SLATEC" -> Gsl_rng.SLATEC
		| "TAUS" -> Gsl_rng.TAUS
		| "TAUS_2" -> Gsl_rng.TAUS_2
		| "TAUS_113" -> Gsl_rng.TAUS_113
		| "TRANSPUTER" -> Gsl_rng.TRANSPUTER
		| "TT800" -> Gsl_rng.TT800
		| "UNI" -> Gsl_rng.UNI
		| "UNI32" -> Gsl_rng.UNI32
		| "VAX" -> Gsl_rng.VAX
		| "WATERMAN14" -> Gsl_rng.WATERMAN14
		| _ -> Gsl_rng.ZUF
	in
	let rng = Gsl_rng.make rngtype in
	let sd = (aod elem "RandomSeed" "AUTO") in
	let randseed = 
	(
		match sd with
		"TIME" -> Nativeint.of_float (Unix.time ())
		| "AUTO" -> 
			Random.self_init ();
			Random.nativeint 1073741823n	(*= 2^30 - 1*)
		| _ -> 
			try Nativeint.of_string (Xml.attrib elem "RandomSeed")
			with Failure _ -> failwith "RandomSeed must be integer."
	) in
	Gsl_rng.set rng randseed;
	rng
;;

